/**
 * HighlightSource Class (HSource)
 * This Object can be deSerialized to HRange.
 * Also it has the ability for persistence.
 */

import type { DomMeta, HookMap, DomNode, SelectionDirection } from '../../types';
import HighlightRange from '../range/index';
import { getTextChildByOffset } from './dom';

class HighlightSource {
    startMeta: DomMeta;

    endMeta: DomMeta;

    text: string;

    id: string;

    extra?: unknown;

    __isHighlightSource: unknown;

    direction: SelectionDirection;

    constructor(
        startMeta: DomMeta,
        endMeta: DomMeta,
        text: string,
        id: string,
        direction: SelectionDirection,
        extra?: unknown,
    ) {
        this.startMeta = startMeta;
        this.endMeta = endMeta;
        this.text = text;
        this.id = id;
        this.__isHighlightSource = {};
        this.direction = direction;

        if (extra) {
            this.extra = extra;
        }
    }

    deSerialize($root: Document | HTMLElement, hooks: HookMap): HighlightRange {
        let startInfo = getTextChildByOffset($root, this.startMeta.textOffset);
        let endInfo = getTextChildByOffset($root, this.endMeta.textOffset);

        if (!hooks.Serialize.Restore.isEmpty()) {
            const res: DomNode[] = hooks.Serialize.Restore.call(this, startInfo, endInfo) || [];

            startInfo = res[0] || startInfo;
            endInfo = res[1] || endInfo;
        }

        const range = new HighlightRange(startInfo, endInfo, this.text, this.id, this.direction, true);

        return range;
    }
}

export default HighlightSource;
